Attribute VB_Name = "AppExtensions"
'  FILE:       AppEntensions.bas
'              Microsoft TPC-H Kit Ver. 2.7.0-1005
'              Copyright Microsoft, 2008
'              All Rights Reserved
'
'
'  PURPOSE:    Application Extensions module
'  Contact:    Reshma Tharamal (reshmat@microsoft.com)
'
' Contains all the procedures to load application extensions in
' the workspace in the Tools menu and Shell the application
Option Explicit

Private Const mstrModuleName As String = "AppExtensions."

Private Const msAppExtensionLabel As String = "&Application Extensions"
Public Sub AppExtensionListDirty(lWsp As Long)
    ' Clears the contents of the tag in the application extensions menu
    ' to force a refresh

    If StringEmpty(frmMain.mnuToolsWsp.Tag) Then
        frmMain.mnuToolsWsp.Tag = 0
    Else
        If CLng(frmMain.mnuToolsWsp.Tag) = lWsp Then frmMain.mnuToolsWsp.Tag = 0
    End If
    
End Sub

Public Sub RunApplication(sAppCommandLine As String)
    ' Runs the passed in application

    Dim sApp As String
    
    On Error GoTo RunApplicationErr
    
    sApp = SubstituteParameters(sAppCommandLine, CLng(frmMain.mnuToolsWsp.Tag))
    
    Call Shell(sApp, vbNormalFocus)
    
    Exit Sub
    
RunApplicationErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    Call ShowError(errRunApplicationFailed, OptArgs:=sApp)
    
End Sub
Public Sub AppExtensionOK(sParamName As String)
    ' Runs the application extension selected by the user

    Dim cTempParam As cParameter
    
    On Error GoTo AppExtensionOKErr
    
    Set cTempParam = gcParameters.GetParameterValue(CLng(frmMain.mnuToolsWsp.Tag), sParamName)
    Call RunApplication(cTempParam.ParameterValue)
    
    Set cTempParam = Nothing
    Exit Sub
    
AppExtensionOKErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    Call ShowError(errRunApplicationFailed)
    
End Sub


Public Sub ShowApplicationExtensions()
    ' Re-use the Open Workspace form to display a list of all defined application
    ' extensions to the user. Once the user selects an application and hits OK,
    ' shell the corresponding application.
    
    Dim lWsp As Long
    Dim lIndex As Long
    
    On Error GoTo ShowApplicationExtensionsErr
    
    ' Set the mousepointer to indicate Busy
    Call ShowBusy
    
    Load frmWorkspaceOpen
    
    frmWorkspaceOpen.Caption = msAppExtensionCaption
    frmWorkspaceOpen.lblWorkspaces.Caption = msAppExtensionLabel
    
    ' I use the tag property of the separator bar to store the workspace id.
    If StringEmpty(frmMain.mnuToolsWsp.Tag) Then frmMain.mnuToolsWsp.Tag = 0
    lWsp = CLng(frmMain.mnuToolsWsp.Tag)
    
    frmWorkspaceOpen.lstWorkspaces.Clear
        
    ' Load the application extension information for the workspace
    For lIndex = 0 To gcParameters.ParameterCount - 1
        If gcParameters(lIndex).ParameterType = gintParameterApplication And _
                gcParameters(lIndex).WorkspaceId = lWsp And _
                gcParameters(lIndex).IndOperation <> DeleteOp Then
            
            ' Populate the listbox with the details for this application
            frmWorkspaceOpen.lstWorkspaces.AddItem gcParameters(lIndex).ParameterName
        End If
    Next lIndex
    
    ' Reset the mousepointer
    ShowFree
    
    frmWorkspaceOpen.Show vbModal, frmMain
    
    Exit Sub
    
ShowApplicationExtensionsErr:
    ' Log the VB error code
    LogErrors Errors
    ShowError errUnableToModifyMenu

End Sub

Public Sub ModifyToolsMenu(lWsp As Long)
    ' This procedure adds all the application extensions that have been defined for the
    ' workspace to the tools menu
    Dim lIndex As Long
    Dim lMenuIndex As Long

    On Error GoTo ModifyToolsMenuErr
    
    lMenuIndex = 0
    
    If StringEmpty(frmMain.mnuToolsWsp.Tag) Then frmMain.mnuToolsWsp.Tag = 0
    
    ' I use the tag property of the separator bar to store the workspace id.
    If CLng(frmMain.mnuToolsWsp.Tag) <> lWsp Then
        ' Load the application extension information for the workspace
        For lIndex = 0 To gcParameters.ParameterCount - 1
            If gcParameters(lIndex).ParameterType = gintParameterApplication And _
                    gcParameters(lIndex).WorkspaceId = lWsp And _
                    gcParameters(lIndex).IndOperation <> DeleteOp Then
                lMenuIndex = lMenuIndex + 1
                
                If (lMenuIndex - 1) > frmMain.mnuToolsAppExtension.UBound Then
                    Exit For
                End If
                
                ' Populate the Application Extensions menu with the details for this application
                frmMain.mnuToolsAppExtension(lMenuIndex - 1).Caption = gcParameters(lIndex).ParameterName
                frmMain.mnuToolsAppExtension(lMenuIndex - 1).Tag = gcParameters(lIndex).ParameterValue
                frmMain.mnuToolsAppExtension(lMenuIndex - 1).Visible = True
            End If
        Next lIndex
        
        ' No extensions?
        frmMain.mnuToolsWsp.Visible = (lMenuIndex > 0)
        
        ' Display the more applications menu option if there are more applications
        frmMain.mnuToolsMoreApps.Visible = ((lMenuIndex - 1) > frmMain.mnuToolsAppExtension.UBound)
        
        ' Disable all remaining extension menuitems
        For lIndex = lMenuIndex To frmMain.mnuToolsAppExtension.UBound
            frmMain.mnuToolsAppExtension(lIndex).Visible = False
        Next lIndex
        
        frmMain.mnuToolsWsp.Tag = lWsp
    End If
    
    Exit Sub
    
ModifyToolsMenuErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "ModifyToolsMenu"
    Call WriteError(errUnableToModifyMenu, gstrSource)
    
End Sub

Public Sub ClearToolsMenu(Optional ByVal lWsp As Long = 0)
    ' This procedure clears all the application extensions that have been defined for the
    ' workspace from the tools menu
    Dim lIndex As Long

    On Error GoTo ClearToolsMenuErr
    
    If StringEmpty(frmMain.mnuToolsWsp.Tag) Then frmMain.mnuToolsWsp.Tag = 0
    
    ' I use the tag property of the separator bar to store the workspace id.
    If lWsp = 0 Or CLng(frmMain.mnuToolsWsp.Tag) = lWsp Then
        ' No extensions
        frmMain.mnuToolsWsp.Visible = False
        
        ' Hide the more applications menu option
        frmMain.mnuToolsMoreApps.Visible = False
        
        ' Disable all extension menuitems
        For lIndex = frmMain.mnuToolsAppExtension.LBound To frmMain.mnuToolsAppExtension.UBound
            frmMain.mnuToolsAppExtension(lIndex).Visible = False
        Next lIndex
        
        frmMain.mnuToolsWsp.Tag = 0
    End If

    Exit Sub
    
ClearToolsMenuErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "ClearToolsMenu"
    Call WriteError(errUnableToModifyMenu, gstrSource)
    
End Sub
    

