Attribute VB_Name = "CommonView"
'  FILE:       CommonView.bas
'              Microsoft TPC-H Kit Ver. 2.7.0-1005
'              Copyright Microsoft, 2008
'              All Rights Reserved
'
'
'  PURPOSE:    Module containing common functionality that is
'              specific to the user-interface
'  Contact:    Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit

Private Const mstrModuleName As String = "CommonView."
Public Function LabelGlobals(lngWorkspaceIdentifier As Long) As String
    ' Returns the globals label for the workspace identifier passed in
    ' Basically this is a wrapper around the MakeKeyValid function
    
    LabelGlobals = MakeKeyValid(gintGlobalsLabel, gintGlobalsLabel, lngWorkspaceIdentifier)

End Function
Public Function LabelParameter(lngWorkspaceIdentifier As Long) As String
    ' Returns the parameter label for the workspace identifier passed in
    ' Basically this is a wrapper around the MakeKeyValid function
    
    LabelParameter = MakeKeyValid(gintParameterLabel, gintParameterLabel, lngWorkspaceIdentifier)

End Function

Public Function LabelParamConnection(lngWorkspaceIdentifier As Long) As String
    ' Returns the Connection Strings label for the workspace identifier passed in
    ' Basically this is a wrapper around the MakeKeyValid function
    
    LabelParamConnection = MakeKeyValid(gintParamConnectionLabel, _
            gintParamConnectionLabel, lngWorkspaceIdentifier)

End Function
Public Function LabelConnectionDtl(lngWorkspaceIdentifier As Long) As String
    ' Returns the Connections label for the workspace identifier passed in
    ' Basically this is a wrapper around the MakeKeyValid function
    
    LabelConnectionDtl = MakeKeyValid(gintConnDtlLabel, _
            gintConnDtlLabel, lngWorkspaceIdentifier)

End Function

Public Function LabelBuiltIns(lngWorkspaceIdentifier As Long) As String
    ' Returns the Built In Parameters label for the workspace identifier passed in
    
    LabelBuiltIns = MakeKeyValid(gintParamBuiltInLabel, _
            gintParamBuiltInLabel, lngWorkspaceIdentifier)

End Function

Public Function LabelAppExtension(lngWorkspaceIdentifier As Long) As String
    ' Returns the Application Extensions label for the workspace identifier passed in
    ' Basically this is a wrapper around the MakeKeyValid function
    
    LabelAppExtension = MakeKeyValid(gintParamExtensionLabel, _
            gintParamExtensionLabel, lngWorkspaceIdentifier)

End Function

Public Function LabelGlobalStep(lngWorkspaceIdentifier As Long) As String
    ' Returns the global steps label for the workspace identifier passed in
    ' Basically this is a wrapper around the MakeKeyValid function
    
    LabelGlobalStep = MakeKeyValid(gintGlobalStepLabel, gintGlobalStepLabel, lngWorkspaceIdentifier)

End Function


Public Function GetNodeText(rstStepsInWorkSpace As Recordset, _
    intTypeOfNode As Integer) As String
    
    On Error GoTo GetNodeTextErr
    gstrSource = mstrModuleName & "GetNodeText"
    
    GetNodeText = gstrEmptyString
    
    ' Null checks - this is necessary since in the case of
    ' step nodes, either the step label, step text or step file name
    ' fields may be populated
    If IsNull(rstStepsInWorkSpace(1)) Then
        If IsStep(intTypeOfNode) Then
            ' Check if the step text file name has been populated
            If IsNull(rstStepsInWorkSpace(2)) Then
                ' If the step text file name is empty, the step
                ' text has to be populated
                If IsNull(rstStepsInWorkSpace(3)) Then
                    On Error GoTo 0
                    Err.Raise errStepLabelOrTextOrFileRequired, _
                            gstrSource, _
                            LoadResString(errStepLabelOrTextOrFileRequired)
                Else
                    GetNodeText = rstStepsInWorkSpace(3)
                End If
            Else
                GetNodeText = rstStepsInWorkSpace(2)
            End If
        Else
            On Error GoTo 0
            Err.Raise errMandatoryNodeTextMissing, _
                    gstrSource, _
                    LoadResString(errMandatoryNodeTextMissing)
        End If
    Else
        GetNodeText = rstStepsInWorkSpace(1)
    End If
    
    Exit Function
    
GetNodeTextErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    gstrSource = mstrModuleName & "GetNodeText"
    Err.Raise vbObjectError + errGetNodeTextFailed, _
            gstrSource, _
            LoadResString(errGetNodeTextFailed)
    
End Function
Public Function FindRowInGrid(msRunGrid As MSFlexGrid, _
        lngRowDtValue As Long) As Long
        
    ' Returns the row at which the passed in value occurs
    ' as the rowdata field
    Dim lngIndex As Long
    
    On Error GoTo FindRowInGridErr
    gstrSource = mstrModuleName & "FindRowInGrid"
    
    For lngIndex = 0 To msRunGrid.Rows - 1
        If msRunGrid.RowData(lngIndex) = lngRowDtValue Then
            FindRowInGrid = lngIndex
            Exit For
        End If
    Next lngIndex
    
    If lngIndex >= msRunGrid.Rows Then
        On Error GoTo 0
        Err.Raise errItemNotFound, gstrSource, _
                LoadResString(errItemNotFound)
    End If
    
    Exit Function
    
FindRowInGridErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise errItemNotFound, gstrSource, _
            LoadResString(errItemNotFound)
    
End Function
Public Sub ForceNumeric(txtCtrl As TextBox, KeyAscii As Integer)
    ' Forces the entry of numeric data in the passed
    ' in text box - throws away all other characters
    Dim intKey As Integer
    
    On Error GoTo ForceNumericErr
    
    If Not (KeyAscii >= Asc(0) And KeyAscii <= Asc(9) Or KeyAscii = 8) Then
        KeyAscii = 0
    End If
    
    Exit Sub
    
ForceNumericErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    ' Not a critical error - the user will have to ensure that
    ' numeric data is entered
    
End Sub

Public Function GetRootNode(treTreeView As TreeView) As Node
    
    ' Returns the root node for the passed in tree view control
    Dim nodAnyNode As Node
    
    On Error GoTo GetRootNodeErr
    
    If treTreeView.Nodes.Count > 0 Then
        For Each nodAnyNode In treTreeView.Nodes
            ' Find the root node in the tree and exit
            Set GetRootNode = nodAnyNode.Root
            Exit For
        Next nodAnyNode
        
        If GetRootNode.Key <> GetRootNode.FirstSibling.Key Then
            Set GetRootNode = GetRootNode.FirstSibling
        End If
    Else
        Set GetRootNode = Nothing
    End If

    Exit Function
    
GetRootNodeErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    gstrSource = mstrModuleName & "GetRootNode"
    Err.Raise vbObjectError + errGetRootNodeFailed, gstrSource, _
            LoadResString(errGetRootNodeFailed)
    
End Function
Public Function InList(SearchList As ListBox, _
        Optional ByVal SearchId As Long = 0, _
        Optional ByVal SearchString As String = gstrEmptyString) _
        As Integer
        
    ' Returns the position at which the passed in id/string
    ' occurs in the list box - returns -1 if it is not found
    
    Dim intIndex As Integer
    
    On Error GoTo InListErr
    
    ' Set the default return value to not found
    InList = -1
    
    BugAssert SearchId <> 0 Or Not StringEmpty(SearchString)
    
    For intIndex = 0 To SearchList.ListCount - 1
        If (SearchId > 0 And SearchList.ItemData(intIndex) = SearchId) Or _
            (Not StringEmpty(SearchString) And SearchList.List(intIndex) = SearchString) Then
                InList = intIndex
                Exit For
        End If
    Next intIndex
    
    Exit Function
    
InListErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errInListFailed, gstrSource, _
        LoadResString(errInListFailed)
    
End Function
Public Sub CopyListData(lstCopyFrom As ListBox, _
        lstCopyTo As ListBox)
    ' Copies all the entries from the CopyFrom list box to
    ' the CopyTo list box control
        
    Dim intIndex As Integer
    
    On Error GoTo CopyListDataErr
    
    For intIndex = 0 To lstCopyFrom.ListCount - 1
        lstCopyTo.AddItem lstCopyFrom.List(intIndex)
        lstCopyTo.ItemData(lstCopyTo.NewIndex) = lstCopyFrom.ItemData(intIndex)
    Next intIndex
    
    Exit Sub
    
CopyListDataErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errCopyListFailed, mstrModuleName & "CopyListData", _
        LoadResString(errCopyListFailed)
    
End Sub

Public Sub SelectFirstItem(lstList As ListBox)
    ' Selects the first item in the passed in list box
    
    If lstList.ListCount > 0 Then
        lstList.ListIndex = 0
    End If
    
End Sub

Public Sub SwapListEntries(lstList As ListBox, _
        intSwapFrom As Integer, _
        intSwapTo As Integer)
    ' Swaps the SwapFrom'th and the SwapTo'th entries in the
    ' passed in list box control
    
    Dim strTemp As String
    Dim lngTempIndex As Long
    
    On Error GoTo SwapListEntriesErr
    
    ' Stores the list and itemdata properties at the SwapTo
    ' position in temporary variables and then does the swapping
    
    If intSwapFrom <> intSwapTo Then
        strTemp = lstList.List(intSwapTo)
        lngTempIndex = lstList.ItemData(intSwapTo)
        
        lstList.List(intSwapTo) = lstList.List(intSwapFrom)
        lstList.ItemData(intSwapTo) = lstList.ItemData(intSwapFrom)
            
        lstList.List(intSwapFrom) = strTemp
        lstList.ItemData(intSwapFrom) = lngTempIndex
    End If
    
    Exit Sub
    
SwapListEntriesErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errCopyListFailed, mstrModuleName & "SwapListEntries", _
        LoadResString(errCopyListFailed)
    
End Sub

Public Sub CentreControlInContainerVertical( _
        ctrToCentre As Control, ctrContainer As Control)

    On Error Resume Next
    
    ctrToCentre.Top = ctrContainer.Top + _
        (ctrContainer.Height - ctrToCentre.Height) / 2
    
End Sub
Public Function StepNode(strNodeKey As String) As Boolean
        
    ' Returns true if the node key belongs to a step node
    ' Returns False otherwise
    
    On Error GoTo StepNodeErr
    
    If strNodeKey = gstrEmptyString Then
        On Error GoTo 0
        Err.Raise vbObjectError + errMandatoryParameterMissing, _
                gstrSource, _
                LoadResString(errMandatoryParameterMissing)
    End If
    
    StepNode = InStr(strNodeKey, gstrGlobalStepPrefix) > 0 Or _
            InStr(strNodeKey, gstrManagerStepPrefix) > 0 Or _
            InStr(strNodeKey, gstrWorkerStepPrefix) > 0
        
    Exit Function

StepNodeErr:
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errStepNodeFailed, _
            gstrSource, _
            LoadResString(errStepNodeFailed)

End Function

Public Function GetOption(optOptions As Object) As Integer
    
    Dim intIndex As Integer

    On Error GoTo GetOptionErr
    gstrSource = mstrModuleName & "GetOption"
    
    For intIndex = optOptions.LBound To optOptions.UBound
        If optOptions(intIndex).Value Then
            GetOption = intIndex
            Exit Function
        End If
    Next intIndex
    
    ' None of the options were true!
    On Error GoTo 0
    Err.Raise vbObjectError + errNoTrueOption, _
            gstrSource, _
            LoadResString(errNoTrueOption)
    
    Exit Function
    
GetOptionErr:
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errGetOptionFailed, _
            gstrSource, _
            LoadResString(errGetOptionFailed)
        
End Function

Public Sub SetOption(optOptions As Object, intIndex As Integer)
    
    On Error GoTo SetOptionErr
    
    optOptions(intIndex).Value = True
    Exit Sub
    
SetOptionErr:
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errSetOptionFailed, gstrSource, _
        LoadResString(errSetOptionFailed)
        
End Sub
Public Function TextEntered(txtCheck As TextBox) As Boolean

    ' Returns True if the passed in text box does not contain
    ' any text - makes sure that the text entered does not contain
    ' just spaces and line feed characters
    
    Dim strTemp As String
    
    On Error GoTo TextEnteredErr
    
    strTemp = txtCheck.Text
    
    Do While Not StringEmpty(strTemp)
        
        ' Replace all line feeds with spaces - these will be
        ' stripped out later
        If InStr(1, strTemp, vbCr & vbLf) > 0 Then
            Mid(strTemp, InStr(strTemp, vbCr & vbLf)) = Space(1)
        ElseIf InStr(1, strTemp, vbLf) > 0 Then
            Mid(strTemp, InStr(strTemp, vbLf)) = Space(1)
        ElseIf InStr(1, strTemp, vbCr) > 0 Then
            Mid(strTemp, InStr(strTemp, vbCr)) = Space(1)
        Else
            Exit Do
        End If
        
    Loop
    
    TextEntered = Not StringEmpty(Trim(strTemp))
    
    Exit Function
    
TextEnteredErr:
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errTextEnteredFailed, _
            mstrModuleName & "TextEntered", _
            LoadResString(errTextEnteredFailed)
        
End Function
Public Sub SetEnabledForCtlArray(ctrArray As Object, blnEnabled As Boolean)
    
    Dim intIndex As Integer

    On Error GoTo SetEnabledForCtlArrayErr
    
    For intIndex = ctrArray.LBound To ctrArray.UBound
        ctrArray(intIndex).Enabled = blnEnabled
    Next intIndex
    
    Exit Sub
    
SetEnabledForCtlArrayErr:
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errSetEnabled, _
            mstrModuleName & "SetEnabledForCtlArray", _
            LoadResString(errSetEnabled)
        
End Sub
Public Sub SetValueForCtlArray(ctrArray As Object, iValue As CheckBoxConstants)
    
    Dim intIndex As Integer

    On Error GoTo SetValueForCtlArrayErr
    
    For intIndex = ctrArray.LBound To ctrArray.UBound
        ctrArray(intIndex).Value = iValue
    Next intIndex
    
    Exit Sub
    
SetValueForCtlArrayErr:
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errSetEnabled, _
            mstrModuleName & "SetValueForCtlArray", _
            LoadResString(errSetEnabled)

End Sub


Public Sub InitCtlArray(ctrArray As Object, vntValue As Variant)
    
    Dim intIndex As Integer

    On Error GoTo InitCtlArrayErr
    
    For intIndex = ctrArray.LBound To ctrArray.UBound
        ctrArray(intIndex).Text = vntValue
    Next intIndex
    
    Exit Sub
    
InitCtlArrayErr:
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errInitializeStepFormFailed, _
            gstrSource, _
            LoadResString(errInitializeStepFormFailed)
        
End Sub
Sub SetVisibleProperty(ctrToDisable As Control, blnValue As Boolean)

    On Error GoTo SetVisiblePropertyErr
    
    ctrToDisable.Enabled = blnValue
    ctrToDisable.Visible = blnValue
    
    Exit Sub
    
SetVisiblePropertyErr:
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errSetVisiblePropertyFailed, _
            gstrSource, _
            LoadResString(errSetVisiblePropertyFailed)
        
End Sub
Public Sub ClearFormAndSetFocus(frmToClear As Form, _
    ctrToSetFocus As Control)
    
    On Error GoTo ClearFormAndSetFocusErr
    
    Call ClearForm(frmToClear)
    ctrToSetFocus.SetFocus
    
    Exit Sub
    
ClearFormAndSetFocusErr:
    ' None of the errors raised in this function are critical
    'Display the error and continue
    Call LogErrors(Errors)
    BugMessage "ClearFormAndSetFocus errored : Resuming..."
    Resume Next
    
End Sub

Public Sub ClearForm(frmToClear As Form)

    Dim intIndex As Integer
    Dim ctlControl As Control
    Dim optOption As OptionButton

    ' The errors raised in this function are not critical
    On Error GoTo ClearFormErr
        
    For intIndex = 0 To frmToClear.Controls.Count - 1
    
        If TypeName(frmToClear.Controls(intIndex)) <> "ImageList" Then
            If frmToClear.Controls(intIndex).Enabled Then
                If TypeOf frmToClear.Controls(intIndex) Is TextBox Then
                    frmToClear.Controls(intIndex).Text = gstrEmptyString
                ElseIf TypeOf frmToClear.Controls(intIndex) Is CheckBox Then
                    ' Clear it
                    frmToClear.Controls(intIndex).Value = 0
                End If
            End If
        End If
        
    Next intIndex
    Exit Sub
    
ClearFormErr:
    'Log the error and continue
    Call LogErrors(Errors)
    BugMessage "ClearForm errored : Resuming..."
    Resume Next
    
End Sub

Sub HighlightText(ctlToHighlight As Control)
        
    'Highlights the text in a textbox
    
    On Error GoTo HighlightTextErr
    
    If TypeOf ctlToHighlight Is TextBox Then
        ctlToHighlight.SelStart = 0
        ctlToHighlight.SelLength = Len(ctlToHighlight)
    End If
    Exit Sub

HighlightTextErr:
    'Log the error and continue
    Call LogErrors(Errors)
    BugMessage "HighlightText errored : Resuming..."
    Resume Next
    
End Sub

Function ItemSelectedInTree(treStepView As TreeView) As String

    ' Returns the key for the tree view node selected by the user.
    ' If none has been selected, returns an empty string

    If treStepView.SelectedItem Is Nothing Then
        ItemSelectedInTree = gstrEmptyString
    Else
        ItemSelectedInTree = treStepView.SelectedItem.Key
    End If

End Function


