Attribute VB_Name = "Step"
'  FILE:       Step.bas
'              Microsoft TPC-H Kit Ver. 2.7.0-1005
'              Copyright Microsoft, 2008
'              All Rights Reserved
'
'
'  PURPOSE:    Contains functionality to add, modify, delete
'              and edit steps. Also functions to customize and initialize
'              the step form based on the step type, etc.
'  Contact:    Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit

' Used to indicate the source module name when errors
' are raised by this module
Private Const mstrModuleName As String = "Step."

Private Const mintMinNumProcesses As Integer = 1

Public cCreateStep As cStep
Private Sub ApplyChanges()

    ' Commits the changes made by the user

    On Error GoTo ApplyChangesErr
    
    If frmSteps.Tag = gstrInsertMode Then
        ' Call the insert functions for each of the different
        ' step types
        If IsGlobal(StepForm:=frmSteps) Then
            Call GlobalStepNew
        ElseIf IsManager(StepForm:=frmSteps) Then
            Call ManagerNew
        Else
            Call WorkerNew
        End If
        ' All subsequent Apply and OK commands will update the step
        frmSteps.Tag = gstrUpdateMode
    Else
        ' Update mode: Call the update functions for each of the
        ' step types
        If IsGlobal(StepForm:=frmSteps) Then
            Call GlobalStepModify
        ElseIf IsManager(StepForm:=frmSteps) Then
            Call ManagerModify
        Else
            Call WorkerModify
        End If
    End If
    
    Exit Sub
    
ApplyChangesErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    ' Reset the mousepointer
    Call ShowFree
    gstrSource = mstrModuleName & "ApplyChanges"
    Err.Raise errUnableToApplyChanges, gstrSource, _
            LoadResString(errUnableToApplyChanges)
    
End Sub
Public Function ConfirmDeleteOrMove( _
        ByVal DeleteStep As cStep, _
        ByVal StepKey As String) As Boolean
    ' Checks if the step node passed in has sub-steps
    ' If so, prompts the user whether all steps should be
    ' deleted/moved. Returns True if the user says Yes.
        
    On Error GoTo ConfirmDeleteOrMoveErr
    
    Dim strMessage As String
    Dim strTitle As String
    Dim intDelete As Integer
    
    ConfirmDeleteOrMove = False
    
    If frmMain.treStepView.Nodes(StepKey).Children > 0 Then
        If DeleteStep Is Nothing Then
            strMessage = "The step you are about to delete " & _
                    "has sub-steps associated with it." & _
                    vbCr & _
                    "Delete all sub-steps?"
            strTitle = "Confirm step delete"
        Else
            strMessage = "The step you are about to cut " & _
                    "has sub-steps associated with it." & _
                    vbCr & _
                    "Move all sub-steps?"
            strTitle = "Confirm move"
        End If
        intDelete = MsgBox(strMessage, _
                vbOKCancel + vbExclamation + vbDefaultButton2 + vbApplicationModal, _
                strTitle)
        If intDelete = vbCancel Then
            ConfirmDeleteOrMove = False
        Else
            ConfirmDeleteOrMove = True
        End If
    Else
        ConfirmDeleteOrMove = True
    End If
    
    Exit Function
    
ConfirmDeleteOrMoveErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    gstrSource = mstrModuleName & "ConfirmDeleteOrMove"
    Err.Raise vbObjectError + errConfirmDeleteOrMoveFailed, _
            gstrSource, _
            LoadResString(errConfirmDeleteOrMoveFailed)

End Function
Public Sub CustomizeStepForm(intTypeOfStep As Integer, _
        ByVal lngWorkspace As Long)

    ' Customizes the step properties form based on the type of
    ' step (global, manager or worker) being created/modified
    
    On Error GoTo CustomizeStepFormErr
    gstrSource = mstrModuleName & "CustomizeStepForm"
    
    ' Display the customized step definition screen for all
    ' manager steps - hide it for worker and global steps
    If intTypeOfStep = gintManagerStep Then
        frmSteps.sstStepProperties.TabVisible(gintDefinition) = False
        frmSteps.sstStepProperties.TabVisible(gintExecution) = False
    Else
        frmSteps.sstStepProperties.TabVisible(gintMgrDefinition) = False
    End If
        
    ' Initialization function b4 constraints can function
    Call ConstraintInit
    
    If intTypeOfStep = gintGlobalStep Then
        ' The step being processed is a global step
        
        ' Disable the tabs for creation of pre- and post-
        ' execution steps
        frmSteps.sstStepProperties.TabVisible(gintPreExecutionSteps) = False
        frmSteps.sstStepProperties.TabVisible(gintPostExecutionSteps) = False
        frmSteps.sstStepProperties.TabVisible(gintFileLocations) = False
    
        ' Disable the enabled flag
        frmSteps.chkEnabled.Value = vbUnchecked
        frmSteps.chkEnabled.Enabled = False
        
        ' Hide all the form fields containing the degree of
        ' parallelism information
        Call SetVisibleProperty(frmSteps.lblParallelismDegree, False)
        Call SetVisibleProperty(frmSteps.txtParallelism, False)
        Call SetVisibleProperty(frmSteps.updNumProcesses, False)
        
        ' Disable all the form fields containing continuation
        ' criteria information
        frmSteps.fraContinuationCriteria.Enabled = False
        Call SetEnabledForCtlArray(frmSteps.optOnFailure, False)
        
        ' Disable the fields to create iterators for the step
        Call HideIteratorFields
    Else
        ' The step being processed is a manager or a worker step
    
        ' Disable the global run method options
'        frmSteps.chkRunStepGlobally.Enabled = False
        frmSteps.fraRunGlobalStep.Visible = False
        
        ' and centre the execution details on the tab
        Call CentreControlInContainerVertical(frmSteps.fraStepExecutionDetails, _
            frmSteps.sstStepProperties)
        
        If lngWorkspace = 0 Then
            On Error GoTo 0
            Err.Raise vbObjectError + errWorkspaceIdMandatory, _
                    gstrSource, _
                    LoadResString(errWorkspaceIdMandatory)
        End If
        
        ' Populate the list of all global steps that qualify as
        ' pre or post execution steps
        Call LoadGlobalSteps(frmSteps.lstGlobals(gintPreStep), _
                frmSteps.lstConstraints(gintPreStep), lngWorkspace)
        ' Call a procedure to dynamically enable the toolbar options,
        Call EnableGlobalCommands(frmSteps.lstGlobals(gintPreStep), _
                frmSteps.lstConstraints(gintPreStep), frmSteps.tblConstraintCommands)
        ' Load all the pre-execution steps that have already
        ' been defined to run globally for the workspace
        Call LoadWspPreExecSteps(frmSteps.lstWspGlobals(gintPreStep), lngWorkspace)
        
        Call CopyListData(frmSteps.lstGlobals(gintPreStep), frmSteps.lstGlobals(gintPostStep))
        ' Call a procedure to dynamically enable the toolbar options,
        Call EnableGlobalCommands(frmSteps.lstGlobals(gintPostStep), _
                frmSteps.lstConstraints(gintPostStep), frmSteps.tblConstraintCommands)
        ' Load all the post-execution steps that have already
        ' been defined to run globally for the workspace
        Call LoadWspPostExecSteps(frmSteps.lstWspGlobals(gintPostStep), lngWorkspace)
        
        ' Do not allow the user to edit the list boxes
        ' containing the workspace constraints
        Call SetEnabledForCtlArray(frmSteps.lstWspGlobals, False)
        
        ' Initialize the iterator entry fields
        Call InitializeIterators
        
        If intTypeOfStep = gintManagerStep Then
            ' Customisations specific to manager steps
        
            ' Since the manager step cannot take any action, disable
            ' the step text and text file name fields
'            frmSteps.lblStepTextFile.Enabled = False
'            frmSteps.txtStepTextFileName.Enabled = False
'            frmSteps.cmdBrowseStepText.Enabled = False
'
'            frmSteps.lblStepText.Enabled = False
'            frmSteps.txtStepText.Enabled = False
            
            ' Disable all fields pertaining to execution options
'            frmSteps.fraExecutionMethod.Enabled = False
'            Call SetEnabledForCtlArray(frmSteps.optExecuteUsing, False)
'
'            frmSteps.fraFailureCriteria.Enabled = False
'            Call SetEnabledForCtlArray(frmSteps.optCheckFailureUsing, False)
'
'            frmSteps.fraFailureDetails.Enabled = False
'            frmSteps.txtFailureDetails.Enabled = False
'            frmSteps.cmdBrowseCompareOutput.Enabled = False
'            frmSteps.cmdPreview.Enabled = False
'
'            frmSteps.fraContinuationCriteria.Enabled = False
'            Call SetEnabledForCtlArray(frmSteps.optOnFailure, False)
        
        Else
            ' Customisations specific to worker steps
        End If
        
    End If
    
    ' Call a procedure to change some boilerplate based on the
    ' step type
    Call TypeSensitiveLabels(intTypeOfStep)
    
    Exit Sub
    
CustomizeStepFormErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    gstrSource = mstrModuleName & "CustomizeStepForm"
    Err.Raise vbObjectError + errCustomizeStepFormFailed, _
            gstrSource, _
            LoadResString(errCustomizeStepFormFailed)
    
End Sub

Public Function FindStepPositionDetails(ByRef cStepToInit As cStep, _
        intStepType As Integer) As String
    
    ' Initializes the passed in class variable with the
    ' parent step id and version information and other properties
    ' that depend on the parent step, namely the step level and
    ' sequence
    ' Returns the key for the parent node
    
    Dim cParentStep As cStep
    Dim lngParentStepId As Long
    Dim strParentKey As String
    
    On Error GoTo FindStepPositionDetailsErr
    
    ' Will contain the queried parent step record
    Set cParentStep = New cStep
    
    ' Call a procedure to find the key of the selected node
    strParentKey = FindParentStepKey(intStepType, cStepToInit.WorkspaceId)
    
    ' First initialize the parent step id and parent version number
    lngParentStepId = FindParentStepId(intStepType, strParentKey, cStepToInit.WorkspaceId)
    If lngParentStepId > 0 Then
        Set cParentStep = gcSteps.QueryStep(lngParentStepId)
    Else
        cParentStep.VersionNo = gstrMinVersion
    End If
    
    cStepToInit.ParentStepId = lngParentStepId
    cStepToInit.ParentVersionNo = cParentStep.VersionNo
    
    ' Call a procedure to determine the step level - the queried
    ' parent will be passed in as a parameter
    cStepToInit.StepLevel = FindStepLevel(cParentStep)
    
    ' Call a procedure to determine the step sequence
    cStepToInit.SequenceNo = FindStepSequence(strParentKey)
    
    Set cParentStep = Nothing
    
    FindStepPositionDetails = strParentKey
    Exit Function
    
FindStepPositionDetailsErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    gstrSource = mstrModuleName & "FindStepPositionDetails"
    Err.Raise vbObjectError + errFindPositionFailed, _
            gstrSource, _
            LoadResString(errFindPositionFailed)
    
End Function
Private Function FindStepLevel(ByVal cParent As cStep) As Integer
    
    ' Determines the step level for the step
    
    BugAssert Not cParent Is Nothing, "Invalid argument"
    FindStepLevel = IIf(cParent.StepId > 0, cParent.StepLevel + 1, gintMinLevel)
    
End Function

Public Function FindStepSequence(strParentStepKey As String) _
        As Integer
    ' Determines the sequence no for the step
    ' The step has not been added to the tree as yet, so we need
    ' to determine the parent step id and then see if it has any
    ' sub-steps
    
    Dim nodLastSibling As Node
    Dim nodChild As Node
    Dim lngLastSiblingId As Long
    Dim cSiblingStep As cStep
    
    On Error GoTo FindStepSequenceErr
    
    ' Check if the parent for the step being added has any sub-steps
    If frmMain.treStepView.Nodes(strParentStepKey).Children > 0 Then
        
        ' If so, find the last child for the parent
        Set nodChild = frmMain.treStepView.Nodes(strParentStepKey).Child
        Set nodLastSibling = nodChild.LastSibling
        lngLastSiblingId = MakeIdentifierValid(nodLastSibling.Key)
        
        ' Query the details for the sibling step and increment
        ' it's sequence no by 1
        Set cSiblingStep = gcSteps.QueryStep(lngLastSiblingId)
        FindStepSequence = cSiblingStep.SequenceNo + 1
        Set cSiblingStep = Nothing
    
    Else
        FindStepSequence = gintMinSequenceNo
    End If
    Exit Function
    
FindStepSequenceErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    gstrSource = mstrModuleName & "FindStepSequence"
    Err.Raise vbObjectError + errFindStepSequenceFailed, _
            gstrSource, _
            LoadResString(errFindStepSequenceFailed)
    
End Function
Private Function FindParentStepId(intStepType As Integer, _
    Optional ByVal strParentStepKey As String = gstrEmptyString, _
    Optional ByVal lngWorkspaceId As Long = 0) As Long
    
    ' Returns the parent step id for the step
    
    Dim strParentKey As String
    
    On Error GoTo FindParentStepIdErr
    
    If StringEmpty(strParentStepKey) Then
        ' Call a procedure to find the key of the selected node
        strParentKey = FindParentStepKey(intStepType, lngWorkspaceId)
    Else
        strParentKey = strParentStepKey
    End If
    
    ' This could be a global label, the steps label or
    ' the step id for the selected step (only for non-globals)
    ' Set the parent step id to 0 for labels and make the
    ' identifier valid for all step ids
    If IsLabel(strParentKey) Then
        FindParentStepId = 0
    Else
        FindParentStepId = MakeIdentifierValid(strParentKey)
    End If
    Exit Function
    
FindParentStepIdErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    gstrSource = mstrModuleName & "FindParentStepId"
    Err.Raise vbObjectError + errFindParentStepIdFailed, _
            gstrSource, _
            LoadResString(errFindParentStepIdFailed)
    
End Function

Public Function FindParentStepKey( _
        intStepType As Integer, _
        Optional ByVal lngWorkspace As Long = 0) As String
    ' Returns the key of the parent step node to which the step
    ' is currently being added
    ' Each step will be added as a child of the node selected
    ' by the user for all non-global steps.
    ' Since global steps cannot have sub-steps, they will be
    ' added as children of the global steps label
    Dim strParentNodeKey As String
    Dim lngWorkspaceId As Long
    
    On Error GoTo FindParentStepKeyErr
    
    strParentNodeKey = ItemSelectedInTree(frmMain.treStepView)
    
    ' Initialize the workspace identifier if it hasn't been passed in
    If lngWorkspace = 0 Then
        lngWorkspaceId = WorkspaceSelected(strParentNodeKey)
    Else
        lngWorkspaceId = lngWorkspace
    End If
    
    If intStepType = gintGlobalStep Then
        ' New global steps will be added as children of the
        ' "Global Steps" label
        strParentNodeKey = LabelGlobalStep(lngWorkspaceId)
    Else
        If StringEmpty(strParentNodeKey) Or _
                Not IsManager(StepKey:=strParentNodeKey) Then
            ' The user has selected some other node and is creating
            ' a step using the New Step menu option. Add the new step
            ' to the Steps label in this case
            strParentNodeKey = LabelStep(lngWorkspaceId)
        End If
    End If
        
    FindParentStepKey = strParentNodeKey
    Exit Function
    
FindParentStepKeyErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    gstrSource = mstrModuleName & "FindParentStepKey"
    Err.Raise vbObjectError + errFindPositionFailed, gstrSource, _
            LoadResString(errFindPositionFailed)
    
End Function
Public Sub InitializeStepForm(intTypeOfStep As Integer, lngWorkspaceId As Long)

    ' Initializes all fields on the step properties form
    
    On Error GoTo InitializeStepFormErr
    
    ' Specific initializations for global, manager and worker
    ' steps, based on the TypeOfStep flag
    If intTypeOfStep = gintManagerStep Then
        ' Important to keep the text and corresponding updown
        ' control values in sync always.
        frmSteps.txtMgrParallelism = Trim(Str(mintMinNumProcesses))
        frmSteps.updMgrNumProcesses.Value = frmSteps.txtMgrParallelism
        
        ' Enable the step by default
        frmSteps.chkMgrEnabled.Value = vbChecked
    Else
        frmSteps.txtStepText.Text = gstrEmptyString
        frmSteps.txtStepText.Tag = gstrEntry
        frmSteps.txtStepTextFileName.Tag = gstrEntry
    
        ' Set the option value for arrays of control options
        ' to their lowest indexes
        frmSteps.optExecuteUsing(frmSteps.optExecuteUsing.LBound) = True
        Call SelectExecutionMethod(frmSteps.optExecuteUsing.LBound)
        
        ' Important to keep the text and corresponding updown
        ' control values in sync always.
        frmSteps.txtParallelism = Trim(Str(mintMinNumProcesses))
        frmSteps.updNumProcesses.Value = frmSteps.txtParallelism
        
        ' Enable the step by default
        frmSteps.chkEnabled.Value = vbChecked
    End If
    
    ' Clear the pre- and post execution constraints' list boxes
    frmSteps.lstConstraints(gintPreStep).Clear
    frmSteps.lstConstraints(gintPostStep).Clear
    ' And reset the selection, if any, in the globals' list boxes
    frmSteps.lstGlobals(gintPreStep).ListIndex = -1
    frmSteps.lstGlobals(gintPostStep).ListIndex = -1
        
    If intTypeOfStep <> gintGlobalStep Then
        Call InitCtlArray(frmSteps.txtItNumValues, mintMinIterator)
    End If
    
    ' Start off on the step definition tab
    frmSteps.sstStepProperties.Tab = IIf(intTypeOfStep = gintManagerStep, gintMgrDefinition, gintDefinition)
    
    Exit Sub
    
InitializeStepFormErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errInitializeStepFormFailed, _
            mstrModuleName, _
            LoadResString(errInitializeStepFormFailed)
    
End Sub
Private Sub CopyFormValuesToClass(frmStepForm As Form, ByRef cStepToInit As cStep)
    ' Initializes the passed in step object with the values
    ' entered by the user on the step properties form.
    ' Since the definition and execution information for the
    ' manager step is entered on a different tab from those
    ' for worker and global steps, we have to be very careful
    ' here
        
    On Error GoTo CopyFormValuesToClassErr
    
    ' Copy the workspace identifier and step type
    cStepToInit.WorkspaceId = Val(frmStepForm.lblWorkspaceId)
    cStepToInit.StepType = Val(frmStepForm.lblStepType)
    
    ' Copy the step definition properties
    If IsManager(StepClass:=cStepToInit) Then
        cStepToInit.StepLabel = frmStepForm.txtMgrStepDescription
    Else
        cStepToInit.StepLabel = frmStepForm.txtStepDescription
    
        If frmStepForm.txtStepText.Tag <> gstrPreview Then
            If TextEntered(frmStepForm.txtStepText) Then
                cStepToInit.StepText = frmStepForm.txtStepText.Text
            Else
                cStepToInit.StepText = gstrEmptyString
            End If
        Else
            cStepToInit.StepText = gstrEmptyString
        End If
        
        If frmStepForm.txtStepTextFileName.Tag <> gstrInline Then
            cStepToInit.StepTextFile = frmStepForm.txtStepTextFileName
        Else
            cStepToInit.StepTextFile = gstrEmptyString
        End If
    End If
    
'    cStepToInit.OutputFile = GetFileLocation(gintOutputFile)
'    cStepToInit.LogFile = GetFileLocation(gintLogFile)
'    cStepToInit.ErrorFile = GetFileLocation(gintErrorFile)
        
    ' Important: The index values for the global run method control
    ' array of option buttons (optRunGlobal) begin at 1
    ' The value 0 is used to indicate that the Run Step Globally
    ' flag is unchecked viz. the step qualifies as a pre or post-
    ' execution step for all other steps in the workspace.
    ' Moreover, all steps other than global steps will have the
    ' global run method set to 0
    ' Note that the Ubound and Lbound properties are used to step
    ' within the control array and the values, 0 and 1 aren't
    ' actually hardcoded anywhere.
'    If Not IsGlobal(StepForm:=frmStepForm) Then
'        ' Global step
'        If frmStepForm.chkRunStepGlobally.Value = Checked Then
'            cStepToInit.GlobalRunMethod = GetOption(frmStepForm.optRunGlobal)
'        Else
'            cStepToInit.GlobalRunMethod = gintRunGlobalPreOrPost
'        End If
'    Else
'        ' Properties pertinent to manager and worker steps
'        If IsManager(StepForm:=frmStepForm) Then
'            cStepToInit.DegreeParallelism = frmStepForm.updMgrNumProcesses
'            cStepToInit.EnabledFlag = frmStepForm.chkMgrEnabled
'        Else
'            cStepToInit.DegreeParallelism = frmStepForm.updNumProcesses
'            cStepToInit.EnabledFlag = frmStepForm.chkEnabled
'        End If
'    End If
    
    If IsManager(StepForm:=frmStepForm) Then
        cStepToInit.DegreeParallelism = ValidateParallelism(frmStepForm.txtMgrParallelism, cStepToInit.WorkspaceId)
        cStepToInit.EnabledFlag = frmStepForm.chkMgrEnabled
    ElseIf IsWorker(StepForm:=frmStepForm) Then
        cStepToInit.DegreeParallelism = ValidateParallelism(frmStepForm.txtParallelism, cStepToInit.WorkspaceId)
        cStepToInit.EnabledFlag = frmStepForm.chkEnabled
        
        cStepToInit.ContinuationCriteria = GetOption(frmStepForm.optOnFailure)
    End If
    
    ' Initialize class with execution details for globals and
    ' worker steps
    If Not IsManager(StepForm:=frmStepForm) Then
        cStepToInit.ExecutionMechanism = GetOption(frmStepForm.optExecuteUsing)
        
        If cStepToInit.ExecutionMechanism = gintExecuteODBC Then
            cStepToInit.StartDir = frmStepForm.cboConnect.Text
        Else
            cStepToInit.StartDir = frmStepForm.txtStartDir
        End If
        cStepToInit.FailureDetails = frmStepForm.txtFailureDetails
    End If
    
    ' Initialize class with the iterator name for manager and
    ' worker steps
    If Not IsGlobal(StepForm:=frmStepForm) Then
        cStepToInit.IteratorName = frmSteps.txtIterator
    End If
    
    Exit Sub
    
CopyFormValuesToClassErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "CopyFormValuesToClass"
    On Error GoTo 0
    Err.Raise vbObjectError + errReadFromScreenFailed, _
            gstrSource, _
            LoadResString(errReadFromScreenFailed)
    
End Sub

Public Sub ShowCreateStepForm(intTypeOfStep As Integer, _
        Optional ByVal lngWorkspaceId As Long = 0, Optional cStepRec As cStep = Nothing)

    Dim lngWorkspace As Long
    
    On Error GoTo ShowCreateStepFormErr
    
    If lngWorkspaceId = 0 Then
        lngWorkspace = WorkspaceSelected
    Else
        lngWorkspace = lngWorkspaceId
    End If
    
    ' Set the tag to show that a new record is being created
    frmSteps.Tag = gstrInsertMode
    
    ' Store the workspace identifier and step type as hidden fields
    ' on the Step form. It will be read later by the create
    ' procedure
    If cStepRec Is Nothing Then
        Set cStepRec = New cStep
        cStepRec.StepType = intTypeOfStep
    Else
        BugAssert cStepRec.StepType = intTypeOfStep, "Step types differ!"
    End If
    
    cStepRec.WorkspaceId = lngWorkspace
    Set cCreateStep = cStepRec
    
    frmSteps.lblWorkspaceId = Str(lngWorkspace)
    frmSteps.lblStepType = Str(intTypeOfStep)
    frmSteps.lblStepId = gstrEmptyString
    
    ' Call procedures to customize the Step properties form
    ' depending on the type of step and set default values
    ' to some of the fields on it
    Call CustomizeStepForm(intTypeOfStep, lngWorkspace)
    Call InitializeStepForm(intTypeOfStep, lngWorkspace)

    ' Call a procedure to display the step properties form
    Call ShowStepForm

    Exit Sub
    
ShowCreateStepFormErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errShowCreateStepFormFailed, _
            gstrSource, _
            LoadResString(errShowCreateStepFormFailed)
    
End Sub
Public Sub ShowStepForm()
    
    On Error GoTo ShowStepFormErr
    
    ' Call a procedure to set the taborder for the form
    Call SetStepTabOrder
    
    ' Reset the mousepointer
    ShowFree
    
    frmSteps.Show vbModal, frmMain

    Exit Sub
    
ShowStepFormErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    gstrSource = mstrModuleName & "ShowStepForm"
    Err.Raise vbObjectError + errShowStepFormFailed, _
            gstrSource, _
            LoadResString(errShowStepFormFailed)
    
End Sub
Public Sub DisplayStepForm()
    
    On Error GoTo DisplayStepFormErr
    
    ' Start off on the Step Definition tab
    If frmSteps.sstStepProperties.TabVisible(gintDefinition) Then
        frmSteps.sstStepProperties.Tab = gintDefinition
    ElseIf frmSteps.sstStepProperties.TabVisible(gintMgrDefinition) Then
        frmSteps.sstStepProperties.Tab = gintMgrDefinition
    End If
    
    ' Reset the mousepointer
    ShowFree
    
    Exit Sub
    
DisplayStepFormErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
        
    gstrSource = mstrModuleName & "DisplayStepForm"
    Call ShowError(errDisplayStepFormFailed)
    Unload frmSteps
    
End Sub

Public Sub StepModify(ByRef cStepToModify As cStep, gintStepType As Integer)
    ' This procedure will handle updates to global, manager and
    ' worker steps
    
    Dim cNewStep As cStep
    
    On Error GoTo StepModifyErr
    
    ' Create a new instance of the step and work on it - this
    ' ensures that if the update doesn't go through, the original
    ' step values are preserved in the array
    Set cNewStep = cStepToModify.Clone
    
    ' First check if the iterator values entered by the
    ' user are valid
    Call ValidateItValues(cNewStep)
    
    ' Call a procedure to copy the updated step properties
    ' from the form to the step object
    Call CopyFormValuesToClass(frmSteps, cNewStep)
    
    Call UpdateStep(cNewStep)
    
    ' Commit constraint records for all worker/manager steps
    If cStepToModify.StepType = gintGlobalStep Then
        ' Call a procedure to modify the constraint records
        ' for the global step
        Call CommitGlobalConstraints(frmSteps.chkRunGlobal, cNewStep)
    Else
        ' Call a procedure to commit all pre and post-execution
        ' constraint records, if any were modified by the user
        Call WriteConstraints(frmSteps.lstConstraints(gintPreStep), _
                frmSteps.lstGlobals(gintPreStep), cNewStep)
        Call WriteConstraints(frmSteps.lstConstraints(gintPostStep), _
                frmSteps.lstGlobals(gintPostStep), cNewStep)
    
        ' Call a procedure to save the iterator values
        ' for the step
        Call WriteIterators(cNewStep)
    End If
    
    Set cNewStep = Nothing
    
    Exit Sub
    
StepModifyErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "StepModify"
    ShowError errStepModifyFailed
    On Error GoTo 0
    Err.Raise vbObjectError + errStepModifyFailed, _
            gstrSource, _
            LoadResString(errStepModifyFailed)
    
End Sub
Public Function ProcessStepModify(Optional ByVal lngIdentifier As Long = 0) As cStep

    Dim lngStepId As Long
    Dim cQueryStep As cStep
    
    On Error GoTo ProcessStepModifyErr

    ' Set the tag to show that an existing record is being modified
    frmSteps.Tag = gstrUpdateMode
    
    lngStepId = IIf(lngIdentifier = 0, IdentifierSelected, lngIdentifier)
    
    BugAssert lngStepId > 0, "ProcessStepModify called when an item wasn't selected!"
    
    Set cQueryStep = gcSteps.QueryStep(lngStepId)
    
    Call CopyClassPropertiesToForm(frmSteps, cQueryStep)
        
    ' Customize the step properties form for Worker steps
    Call CustomizeStepForm(cQueryStep.StepType, cQueryStep.WorkspaceId)
    
    ' Call a procedure to load all iterator values for the step
    Call LoadIterators(cQueryStep)
    
    Call LoadConstraintsOnForm(cQueryStep)
    
    frmSteps.sstStepProperties.Tab = IIf(cQueryStep.StepType = gintManagerStep, _
            gintMgrDefinition, gintDefinition)

    Set ProcessStepModify = cQueryStep
    
    Exit Function
    
ProcessStepModifyErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "ProcessStepModify"
    On Error GoTo 0
    Err.Raise vbObjectError + errProcessStepModifyFailed, _
            gstrSource, _
            LoadResString(errProcessStepModifyFailed)
    
End Function
Private Sub LoadConstraintsOnForm(ByVal cStepRec As cStep)
    
    ' Loads all the constraint records for the passed in step
    
    On Error GoTo LoadConstraintsOnFormErr
    
    If cStepRec.StepType = gintGlobalStep Then
        ' Initialize the constraint checkboxes
        Call LoadConstraintsForGlobal(frmSteps.chkRunGlobal, _
                cStepRec.StepId, cStepRec.VersionNo)
        
    Else
        ' Load all pre- and post-execution steps that have been
        ' defined for the step
        Call LoadPreExecutionSteps(frmSteps.lstConstraints(gintPreStep), _
                cStepRec.StepId, cStepRec.VersionNo)
        ' Call a procedure to enable/disable the toolbar options,
        Call EnableConstraintCommands(frmSteps.lstGlobals(gintPreStep), _
                frmSteps.lstConstraints(gintPreStep), _
                frmSteps.tblConstraintCommands)

        Call LoadPostExecutionSteps(frmSteps.lstConstraints(gintPostStep), _
                cStepRec.StepId, cStepRec.VersionNo)
        ' Call a procedure to enable/disable the toolbar options,
        Call EnableConstraintCommands(frmSteps.lstGlobals(gintPostStep), _
                frmSteps.lstConstraints(gintPostStep), _
                frmSteps.tblConstraintCommands)
    
    End If
    
    Exit Sub
    
LoadConstraintsOnFormErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    gstrSource = mstrModuleName & "LoadConstraintsOnForm"
    Err.Raise vbObjectError + errLoadConstraintsOnFormFailed, _
            gstrSource, _
            LoadResString(errLoadConstraintsOnFormFailed)
    
End Sub

Private Sub StepReset()
    
    On Error Resume Next
    
    ' Unload the form once the updates are completed
    Unload frmSteps

End Sub

Public Sub StepNew(ByRef cNewStep As cStep, _
        intStepType As Integer)
    ' This procedure initializes the step object with all
    ' the step properties and calls a procedure to insert
    ' the step

    Dim strParentNodeKey As String
    
    On Error GoTo StepNewErr
    
    ' Initialize the version number to 0
    cNewStep.VersionNo = gstrMinVersion
    
    ' Call a common procedure to initialize class with step properties
    Call CopyFormValuesToClass(frmSteps, cNewStep)
    
    ' Now determine the properties that will determine the position
    ' of the step in the hierarchy
    ' Note: The workspace id in the step object has already been
    ' initialized by the above procedure.
    If cNewStep.SequenceNo = 0 Then
        strParentNodeKey = FindStepPositionDetails(cNewStep, intStepType)
    Else
        If cNewStep.ParentStepId = 0 Then
            strParentNodeKey = LabelStep(cNewStep.WorkspaceId)
        Else
            strParentNodeKey = MakeKeyValid(cNewStep.ParentStepId, gintManagerStep)
        End If
    End If
    
    Call InsertStep(cNewStep, strParentNodeKey, iCreate:=Insert)
    
    ' Store the constraint records for all steps
    If cNewStep.StepType = gintGlobalStep Then
        ' Call a procedure to commit all constraint records for the global step
        Call CommitGlobalConstraints(frmSteps.chkRunGlobal, cNewStep)
    Else
        ' Call a procedure to commit all pre and post-execution
        ' constraint records, if any were created by the user
        Call WriteConstraints(frmSteps.lstConstraints(gintPreStep), _
                frmSteps.lstGlobals(gintPreStep), cNewStep)
        Call WriteConstraints(frmSteps.lstConstraints(gintPostStep), _
                frmSteps.lstGlobals(gintPostStep), cNewStep)
                
        ' Call a procedure to save the iterator values for the step
        Call WriteIterators(cNewStep)
    End If
    
    Exit Sub
    
StepNewErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "StepNew"
    ShowError errStepNewFailed
    On Error GoTo 0
    Err.Raise vbObjectError + errStepNewFailed, gstrSource, LoadResString(errStepNewFailed)
    
End Sub
Public Sub StepsApply()

    ' Called whenever the user selects the Apply command
    ' Commits the changes made by the user

    On Error GoTo StepsApplyErr
    
    ' Set the mousepointer to indicate Busy
    Call ShowBusy
    
    ' Call a procedure to call the appropriate insert/update
    ' function
    Call ApplyChanges
    
    ' Reset the mousepointer
    Call ShowFree
    
    Exit Sub
    
StepsApplyErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    ' Reset the mousepointer
    Call ShowFree
    
    ' If the insert or update operations errored out, display the
    ' step form again to let the user enter valid values
    Call DisplayStepForm
    
End Sub
Public Sub StepsOK()

    ' The Ok button selected event will be triggered whenever
    ' the user hits enter, since it is the Default command.

    On Error GoTo StepsOKErr
    
    ' Set the mousepointer to indicate Busy
    Call ShowBusy
    
    ' Call a procedure to carry out the inserts/updates
    Call ApplyChanges
    
    ' Reset the current contents
    Call StepReset
    
    ' Reset the mousepointer
    Call ShowFree
    Exit Sub
    
StepsOKErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    ' Reset the mousepointer
    Call ShowFree
    
    ' If the insert or update operations errored out, display the
    ' step form again to let the user enter valid values
    Call DisplayStepForm
    
End Sub

Public Sub StepsCleanup()

    'Do any necessary cleanup here
    
    On Error GoTo StepsCleanupErr
    
    Call ConstraintCleanup
    
    ' Reset the mousepointer
    Call ShowFree
    Exit Sub

StepsCleanupErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    Resume Next
    
End Sub

Private Sub TypeSensitiveLabels(intStepType As Integer)
    ' Changes some labels on the step properties form to display
    ' labels depending on the type of step being shown

    Dim strTemp As String
    
    ' None of the errors in this function can be really critical -
    ' we can safely resume
    On Error Resume Next
    
    If frmSteps.Tag = gstrInsertMode Then
        strTemp = "Create "
    Else
        strTemp = "Modify "
    End If
    
    Select Case intStepType
        Case gintGlobalStep
'            frmSteps.lblStepLabel = frmSteps.lblStepLabel.Caption & _
'                    " (Optional)"
            frmSteps.Caption = strTemp & "Global Step"
                    
        Case gintManagerStep
'            frmSteps.lblStepLabel = frmSteps.lblStepLabel.Caption & _
'                    " (Required)"
            frmSteps.Caption = strTemp & "Manager Step"
        
        Case gintWorkerStep
'            frmSteps.lblStepLabel = frmSteps.lblStepLabel.Caption & _
'                    " (Optional)"
            frmSteps.Caption = strTemp & "Worker Step"
        
    End Select
    
End Sub
