VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cArrSteps"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'  FILE:       cArrSteps.cls
'              Microsoft TPC-H Kit Ver. 2.7.0-1005
'              Copyright Microsoft, 2008
'              All Rights Reserved
'
'
'  PURPOSE:    Implements an array of cStep objects.
'              Type-safe wrapper around cNodeCollections.
'              Also contains additional functions to update parent version
'              on substeps, validation functions, etc.
'  Contact:    Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit

Private mcarrSteps As cNodeCollections

' Used to indicate the source module name when errors
' are raised by this class
Private mstrSource As String
Private Const mstrModuleName As String = "cArrSteps."

Public Sub Unload(lngStepToDelete As Long)

    Dim lngDeleteElement As Long
    Dim cUnloadStep As cStep
    
    lngDeleteElement = QueryStepIndex(lngStepToDelete)
    Set cUnloadStep = QueryStep(lngStepToDelete)
    
    ' First unload all iterators for the step
    Call cUnloadStep.UnloadIterators

    ' Unload the step from the collection
    Call mcarrSteps.Unload(lngDeleteElement)
    
End Sub
Public Sub Modify(cModifiedStep As cStep)
    
    Dim iAppend As Integer
    Dim sLabel As String
    
    On Error GoTo ModifyErr

    iAppend = 0
    sLabel = cModifiedStep.StepLabel

    Validate cModifiedStep
    
    Call mcarrSteps.Modify(cModifiedStep)
    
    Exit Sub
    
ModifyErr:
    ' If the error raised by the add function is due to a duplication
    ' of the step label, then try to generate a unique label
    If Err.Number - vbObjectError = errStepLabelUnique Then
        iAppend = iAppend + 1
        cModifiedStep.StepLabel = sLabel & CStr(iAppend)
        ' Try to insert the step record again
        Resume
    End If
    
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errModifyStepFailed, _
            gstrSource, _
            LoadResString(errModifyStepFailed)

End Sub
Public Sub UpdateParentVersion(ByVal lngStepId As Long, _
        ByVal strNewVersion As String, _
        ByVal strOldVersion As String, _
        ByVal intStepType As Integer)

    ' Does all the processing needed to update the parent version
    ' number on all the sub-steps for a given step
    ' It updates the parent version no in the database for all
    ' sub-steps of the passed in step id
    ' It also updates the parent version number on all sub-steps
    ' in the array to the new version passed in
    
    Dim lngIndex As Long
    Dim cUpdateStep As cStep
    
    On Error GoTo UpdateParentVersionErr
    
    If intStepType <> gintManagerStep Then
        ' Only a manager can have sub-steps - if the passed
        ' in step is not a manager, exit
        Exit Sub
    End If
    
    ' For all steps in the array
    For lngIndex = 0 To mcarrSteps.Count - 1
    
        Set cUpdateStep = mcarrSteps(lngIndex)
        
        ' If the current step is a sub-step of the passed in step
        If cUpdateStep.ParentStepId = lngStepId And _
                cUpdateStep.ParentVersionNo = strOldVersion And _
                Not cUpdateStep.ArchivedFlag Then
            
            ' Update the parent version number for the sub-step
            ' in the array
            cUpdateStep.ParentVersionNo = strNewVersion
            
            ' Update the parent version number for the sub-step
            ' in the array
            Call Modify(cUpdateStep)
        
        End If
    Next lngIndex
    
    Exit Sub
    
UpdateParentVersionErr:
    LogErrors Errors
    mstrSource = mstrModuleName & "UpdateParentVersion"
    On Error GoTo 0
    Err.Raise vbObjectError + errUpdateParentVersionFailed, _
            mstrSource, _
            LoadResString(errUpdateParentVersionFailed)
        
End Sub
Private Sub Validate(cCheckStep As cStep)

    ' Step validations that depend on other steps in the collection
   
    Dim lngIndex As Long
    
    ' Ensure that the step label is unique in the workspace
    For lngIndex = 0 To mcarrSteps.Count - 1
    
        ' If the current step is a sub-step of the passed in step
        If mcarrSteps(lngIndex).WorkspaceId = cCheckStep.WorkspaceId And _
                mcarrSteps(lngIndex).StepLabel = cCheckStep.StepLabel And _
                mcarrSteps(lngIndex).StepId <> cCheckStep.StepId And _
                mcarrSteps(lngIndex).IndOperation <> DeleteOp Then
            On Error GoTo 0
            Err.Raise vbObjectError + errStepLabelUnique, _
                    mstrModuleName & "Validate", _
                    LoadResString(errStepLabelUnique)
        End If
    Next lngIndex
    
End Sub

Public Sub ValidateStep(cCheckStep As cStep)

    On Error GoTo ValidateStepErr
    
    'Public wrapper for Validate function (2 many Validates)
    Call Validate(cCheckStep)
    
    Exit Sub
    
ValidateStepErr:
    ShowError errStepLabelUnique
    On Error GoTo 0
    Err.Raise vbObjectError + errValidateFailed, _
            gstrSource, _
            LoadResString(errValidateFailed)
End Sub

Private Sub Class_Initialize()
    
    BugMessage "cArrSteps: Initialize event - setting step count to 0"
    Set mcarrSteps = New cNodeCollections

End Sub

Private Sub Class_Terminate()

    BugMessage "cArrSteps: Terminate event triggered"
    Set mcarrSteps = Nothing
    
End Sub

Public Sub Add(ByVal cStepToAdd As cStep)
    
    Dim iAppend As Integer
    Dim sLabel As String
    
    On Error GoTo AddErr

    iAppend = 0
    sLabel = cStepToAdd.StepLabel

    Set cStepToAdd.NodeDB = mcarrSteps.NodeDB
    
    ' Retrieve a unique step identifier
    cStepToAdd.StepId = cStepToAdd.NextStepId
    
    Validate cStepToAdd
    
    ' Call a procedure to add the step record
    Call mcarrSteps.Add(cStepToAdd)
    
    ' Call a procedure to add all iterators for the step
    cStepToAdd.AddAllIterators
    
    Exit Sub
    
AddErr:
    ' If the error raised by the add function is due to a duplication
    ' of the step label, then try to generate a unique label
    If Err.Number - vbObjectError = errStepLabelUnique Then
        iAppend = iAppend + 1
        cStepToAdd.StepLabel = sLabel & CStr(iAppend)
        ' Try to insert the step record again
        Resume
    End If
    
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errInsertStepFailed, _
            gstrSource, _
            LoadResString(errInsertStepFailed)

End Sub
Public Sub Load(cStepToLoad As cStep)

    Call mcarrSteps.Load(cStepToLoad)
    
End Sub
Public Sub SaveStepsInWsp(ByVal lngWorkspace As Long)
    ' Calls a procedure to commit all changes to the steps
    ' in the passed in workspace.
    
    Dim lngIndex As Integer

    ' Find all steps in the array with a matching workspace id
    ' It is important to step in reverse order through the array,
    ' since we delete step records sometimes!
    For lngIndex = mcarrSteps.Count - 1 To 0 Step -1
        If mcarrSteps(lngIndex).WorkspaceId = lngWorkspace Then
        
            ' Call a procedure to commit all changes to the
            ' Step record, if any
            Call CommitStep(mcarrSteps(lngIndex), lngIndex)
            
        End If
    Next lngIndex
    
End Sub
Private Sub CommitStep(ByVal cCommitStep As cStep, _
        ByVal intIndex As Integer)
    ' This procedure checks if any changes have been made to the
    ' passed in Step. If so, it calls the step methods to commit
    ' the changes.
    
    ' First commit all changes to the iterator records for
    ' the step
    cCommitStep.SaveIterators
    
    Call mcarrSteps.Commit(cCommitStep, intIndex)
    
End Sub
Public Sub Delete(lngStepToDelete As Long)
    
    Dim lngDeleteElement As Long
        
    lngDeleteElement = QueryStepIndex(lngStepToDelete)
    Call mcarrSteps.Delete(lngDeleteElement)
    
End Sub
Public Function QueryStepIndex(lngStepId As Long) As Long

    Dim lngIndex As Long

    ' Find the element in the array that corresponds to the
    ' passed in step id - note that while there will be multiple
    ' versions of a step in the database, only one version will
    ' be currently loaded in the array - meaning that the stepid
    ' is enough to uniquely identify a step
    For lngIndex = 0 To mcarrSteps.Count - 1
        If mcarrSteps(lngIndex).StepId = lngStepId Then
            QueryStepIndex = lngIndex
            Exit Function
        End If
    Next lngIndex
    
    ' Raise error that step has not been found
    On Error GoTo 0
    Err.Raise vbObjectError + errStepNotFound, mstrSource, _
        LoadResString(errStepNotFound)
    
End Function

Public Function QueryStep(ByVal lngStepId As Long) As cStep
    
    ' Populates the passed in cStep object with the property
    ' values corresponding to the Step Identifier, lngStepId
    
    Dim lngQueryElement As Integer
        
    lngQueryElement = QueryStepIndex(lngStepId)
    
    ' Initialize the passed in step object to the queried step
    Set QueryStep = mcarrSteps(lngQueryElement)
    
End Function
Public Property Get Item(ByVal Position As Long) As cStep
Attribute Item.VB_UserMemId = 0
    
    ' Returns the element at the passed in position in the array
    If Position >= 0 And Position < mcarrSteps.Count Then
        Set Item = mcarrSteps(Position)
    Else
        On Error GoTo 0
        Err.Raise vbObjectError + errItemDoesNotExist, mstrSource, _
            LoadResString(errItemDoesNotExist)
    End If
    
End Property
Public Property Set Item(ByVal Position As Long, _
        ByVal cStepRec As cStep)
    
    ' Returns the element at the passed in position in the array
    If Position >= 0 And Position < mcarrSteps.Count Then
        Set mcarrSteps(Position) = cStepRec
    Else
        On Error GoTo 0
        Err.Raise vbObjectError + errItemDoesNotExist, mstrSource, _
            LoadResString(errItemDoesNotExist)
    End If
    
End Property
Public Property Set StepDB(vdata As Database)

    Set mcarrSteps.NodeDB = vdata
    
End Property
Public Function SubSteps(ByVal lngStepId As Long, _
        ByVal strVersionNo As String) As Variant
    
    ' Returns a variant containing an array of all the substeps
    ' for the passed in step
    
    Dim intIndex As Integer
    Dim cSubSteps() As cStep
    Dim lngStepCount As Long
    Dim cQueryStep As cStep
    
    On Error GoTo SubStepsErr
    
    lngStepCount = 0
    
    Set cQueryStep = QueryStep(lngStepId)
    
    ' Only a manager can have sub-steps
    If cQueryStep.StepType = gintManagerStep Then
        
        ' For each element in the Steps array
        For intIndex = 0 To mcarrSteps.Count - 1
            ' Check if the parent step id and parent version number
            ' match the passed in step
            If mcarrSteps(intIndex).ParentStepId = lngStepId And _
                    mcarrSteps(intIndex).ParentVersionNo = strVersionNo And _
                    mcarrSteps(intIndex).IndOperation <> DeleteOp Then
                
                ' Increase the array dimension and add the step
                ' to it
                ReDim Preserve cSubSteps(lngStepCount)
                Set cSubSteps(lngStepCount) = mcarrSteps(intIndex)
                lngStepCount = lngStepCount + 1
    
            End If
        Next intIndex
    
    End If
    
    ' Set the return value of the function to the array of
    ' Steps that has been built above
    If lngStepCount = 0 Then
        SubSteps = Empty
    Else
        SubSteps = cSubSteps()
    End If
    
    Exit Function
    
SubStepsErr:
    LogErrors Errors
    mstrSource = mstrModuleName & "SubSteps"
    On Error GoTo 0
    Err.Raise vbObjectError + errSubStepsFailed, _
            mstrSource, _
            LoadResString(errSubStepsFailed)
        
End Function

Public Property Get StepCount() As Integer

    StepCount = mcarrSteps.Count
    
End Property

