VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "cExecuteSM"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Attribute VB_Ext_KEY = "SavedWithClassBuilder" ,"Yes"
Attribute VB_Ext_KEY = "Top_Level" ,"Yes"
#If Not NEW_EXECUTE Then

' This class executes the command that is assigned to the
' Command property and logs the output, error and messages
' to the specified files. It raises an event when the
' command completes execution.
' in sequence and then the step itself. At the end it executes
' the post-execution constraints. Since these steps should always
' be executed in sequence, each step is only fired on the
' completion of the previous step.
Option Explicit

' Used to indicate the source module name when errors
' are raised by this class
Private mstrSource As String
Private Const mstrModuleName As String = "cExecuteSM."

Private mcRDOErrorsSM As cRDOErrorsSM
Private mcVBErrorsSM As cVBErrorsSM

' Properties of the command being executed
Private mstrCommand As String
Private mstrStartDir As String
Private mblnCreateRs As Boolean
Private mblnRunAsyncSM As Boolean
Private mblnCancel As Boolean
Private mintType As ExecutionMethod
Private mdtmStartTime As Date
Private mdtmEndTime As Date
Public StepStatus As InstanceStatus

Private WithEvents mcTimer As cTimerSM
Attribute mcTimer.VB_VarHelpID = -1

'Private Enum ErrorStatus
'    miUnknown = 0
'    miError
'    miSuccess
'End Enum
'
' Instances of the file class for the output file and the
' error and message logs
Private mcOutputFile As cFileSM
Private mcErrorFile As cFileSM
'Private mcLogFile As cFileSM

Private WithEvents mcAsyncProc As cAsyncShell
Attribute mcAsyncProc.VB_VarHelpID = -1
Private WithEvents mconConnection As rdoConnection
Attribute mconConnection.VB_VarHelpID = -1
Private WithEvents mrecResults As rdoResultset
Attribute mrecResults.VB_VarHelpID = -1

' Public event to notify the calling function of the
' end time for the command
Public Event Complete(ByVal EndTime As Date, ByVal Elapsed As Long)
Public Event Start(ByVal StartTime As Date)

Private Const msColSeparator As String = "|"

Public Enum InstanceStatus
    gintDisabled = 1
    gintPending
    gintRunning
    gintComplete
    gintFailed
    gintAborted
End Enum

Public Sub Abort()

    On Error GoTo AbortErr

    mblnCancel = True
    
    If mintType = gintExecuteODBC Then
        If mblnCreateRs Then
            ' The rdoResultset object has a Cancel method
            mrecResults.Cancel
        Else
            ' If we are executing an action query, use the query
            ' timeout event to set the Cancel property to true.
        End If
    Else
        If mblnRunAsyncSM Then
            ' Without start
            If mcAsyncProc Is Nothing Then
                ' Looks like the process hasn't started
                ' executing yet
            Else
                mcAsyncProc.Abort
            End If
        Else
            ' Cannot terminate a synchronously executing shell
            ' process - on the other hand, we shall never
            ' execute a process (Shell or ODBC) synchronously!
            Call WriteError(errTerminateProcessFailed, mstrSource)
        End If
    End If
    
    Exit Sub

AbortErr:
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errProgramError, mstrModuleName & "Abort", _
        LoadResString(errProgramError)

End Sub

Private Sub LogOutput()
    
    Dim cIndex As rdoColumn
    Dim sField As String
    Dim sColDim() As Long
    Dim lTemp As Long
    
    On Error GoTo LogOutputErr
    
    ' Since atleast the first set of results has been
    ' populated, write them to the output file
    ' Loop through the remaining result sets
    Do
        If mrecResults.rdoColumns.Count > 0 Then
'            mcOutputFile.WriteLine "Writing results of : " & mrecResults.Name

            ReDim sColDim(0 To mrecResults.rdoColumns.Count - 1) As Long
            lTemp = 0
            
            ' Log the column names
            For Each cIndex In mrecResults.rdoColumns
                sColDim(lTemp) = IIf(cIndex.Size > Len(cIndex.Name), cIndex.Size, Len(cIndex.Name))
                BugAssert sColDim(lTemp) - Len(cIndex.Name) >= 0
                sField = cIndex.Name & Space(sColDim(lTemp) - Len(cIndex.Name))
                mcOutputFile.WriteField sField & msColSeparator
                lTemp = lTemp + 1
            Next cIndex
            mcOutputFile.WriteLine gstrEmptyString
            
            lTemp = 0
            For Each cIndex In mrecResults.rdoColumns
                sField = String(sColDim(lTemp), "-")
                mcOutputFile.WriteField sField & msColSeparator
                lTemp = lTemp + 1
            Next cIndex
            mcOutputFile.WriteLine gstrEmptyString
            
            Do While Not mrecResults.EOF
                lTemp = 0
                For Each cIndex In mrecResults.rdoColumns
                    sField = CStr("" & cIndex.Value) & Space(sColDim(lTemp) - Len(CStr("" & cIndex.Value)))
                    mcOutputFile.WriteField sField & msColSeparator
                    lTemp = lTemp + 1
                Next cIndex
                mrecResults.MoveNext
                mcOutputFile.WriteLine gstrEmptyString
            Loop
        End If

        mcOutputFile.CloseFile

    Loop While mrecResults.MoreResults
    
    Exit Sub
    
LogOutputErr:
            
    ' Log all the errors generated by the ODBC layer, if any
    mcRDOErrorsSM.LogRDOErrors
    
    ' Log the VB errors
    mcVBErrorsSM.LogVBErrors

End Sub
Private Sub ProcessResultset()

    On Error GoTo ProcessResultsetErr
    
'    If iErrStatus = miError Then
    If ExecutionFailed() Then
        StepStatus = gintFailed
        mcRDOErrorsSM.LogRDOErrors
    Else
        ' Do not overwrite a failed status if any of the previous resultsets
        ' in the ODBC batch errored out
        If StepStatus <> gintFailed Then StepStatus = gintComplete
        
        mcRDOErrorsSM.LogRDOMessages
        
'        mcOutputFile.WriteLine "Part of the step completed at " & mdtmEndTime
'
        If mblnCreateRs And Not mrecResults Is Nothing Then
            Call LogOutput
            If Not mrecResults.StillExecuting Then
                mrecResults.Close
            End If
        End If
    End If
    
ExecComplete:
    If mconConnection.StillExecuting = False Then
        ' Cleanup
        Call ExecutionComplete
    End If

    Exit Sub

ProcessResultsetErr:

    ' Log all the errors generated by the ODBC layer, if any
    mcRDOErrorsSM.LogRDOErrors
    
    ' Log the VB errors
    mcVBErrorsSM.LogVBErrors
    GoTo ExecComplete
    
End Sub
Public Property Let StartDir(ByVal vdata As String)
    
    mstrStartDir = vdata

End Property
Public Property Get StartDir() As String

    StartDir = mstrStartDir
    
End Property
Private Sub ExecutionComplete()
    Dim lElapsed As Long
    
    On Error GoTo ExecutionCompleteErr
    
    lElapsed = DateDiff(IN_SECONDS, mdtmStartTime, mdtmEndTime)
    
    ' Make sure that the end time was initialized by the ODBC and
    ' the Shell processes - we don't do it in this common function
    ' since we want the time to be as accurate as possible
    BugAssert mdtmEndTime <> gdtmEmpty, "End time uninitialized."
    
    If mintType = gintExecuteODBC Then
        ' Close the connection opened by this instance
        mconConnection.Close
        Set mconConnection = Nothing
    Else
        Set mcAsyncProc = Nothing
    End If
    
    RaiseEvent Complete(mdtmEndTime, lElapsed)

    Exit Sub

ExecutionCompleteErr:
    ' Was it the Connection close that errored out?
    mcRDOErrorsSM.LogRDOErrors
    
    ' Log the VB errors
    mcVBErrorsSM.LogVBErrors
    
    ' Raise a Complete event anyway
    RaiseEvent Complete(mdtmEndTime, lElapsed)
    
End Sub
Public Property Get ExecutionMechanism() As ExecutionMethod

    ExecutionMechanism = mintType
    
End Property
Public Property Let ExecutionMechanism(ByVal vdata As ExecutionMethod)
    BugAssert ((vdata = gintExecuteODBC) Or (vdata = gintExecuteShell)), "Exec. method invalid!"
    mintType = vdata
End Property

Public Sub Execute()

    On Error GoTo ExecuteErr
    
    BugAssert (Not StringEmpty(mstrCommand)), "Command invalid!"
    
    If mintType = gintExecuteODBC Then
        Call ExecuteODBC
    Else
        Call ExecuteShell
    End If
    StepStatus = gintRunning

    Exit Sub

ExecuteErr:
    StepStatus = gintFailed
    
    ' Log the RDO errors, if any
    Call mcRDOErrorsSM.LogRDOErrors
    
    ' Log the VB errors
    mcVBErrorsSM.LogVBErrors
    mstrSource = mstrModuleName & "Execute"
    On Error GoTo 0
    Err.Raise vbObjectError + errExecuteStepFailed, mstrSource, _
            LoadResString(errExecuteStepFailed)
    
End Sub
Private Sub ExecuteODBC()

    Dim vntOption As Variant
    Dim sConnect As String
    
    If StringEmpty(mstrStartDir) Then
        mstrStartDir = "Uid=sa;pwd=;database=master;dsn=StepMaster;"
    End If
    
    Set mconConnection = gRunEngine.rdoEnvironments(0).OpenConnection("", rdDriverNoPrompt, False, mstrStartDir)
    If mconConnection Is Nothing Then
        BugMessage "'" & mstrCommand & "' could not be executed. Unable to connect to database."
    Else
        If mblnRunAsyncSM Then
            vntOption = rdAsyncEnable
        End If
        BugMessage "Executing: " & mstrCommand
        mconConnection.QueryTimeout = 0
        
        mdtmStartTime = Now
        RaiseEvent Start(mdtmStartTime)
        
        If mblnCreateRs Then
            Set mrecResults = mconConnection.OpenResultset(mstrCommand, , _
                    LockType:=rdConcurReadOnly, Options:=vntOption)
'            DoEvents
        Else
            Call mconConnection.Execute(mstrCommand, Options:=vntOption)
        End If
        
        With mcTimer
            .Interval = 1000
            .Enabled = True
        End With
    End If
    
End Sub
Public Property Let CreateResultset(ByVal vdata As Boolean)
    
    mblnCreateRs = vdata

End Property
Public Property Let RunAsyncSM(ByVal vdata As Boolean)
    
    mblnRunAsyncSM = vdata

End Property

Public Property Get CreateResultset() As Boolean

    CreateResultset = mblnCreateRs
    
End Property
Public Property Get RunAsyncSM() As Boolean

    RunAsyncSM = mblnRunAsyncSM
    
End Property


Private Sub ExecuteShell()

    Dim strExec As String
    Dim strOutput As String
    Dim sOldDir As String
    
    ' Check if the output file can be created/written to
    If OpenFileSM(mcOutputFile.FileName) = 0 Then
        ' Either the directory was invalid or s'thing failed
'        ' Display the error to the user instead of trying
'        ' to log to the file
'        ShowError errUnableToLogOutput, gstrSource, mstrCommand, _
'                DoWriteError:=False
        ' Write the error to the project log
        WriteError errUnableToLogOutput, gstrSource, mstrCommand
    Else
        strOutput = " > " & mcOutputFile.FileName
        ' Close the file since it needs to be available for the
        ' Shell command
        CloseFileSM (mcOutputFile.FileName)
    End If
        
    ' Similar processing for the error file
    If OpenFileSM(mcErrorFile.FileName) = 0 Then
'        ShowError errUnableToLogOutput, gstrSource, mstrCommand, _
'                DoWriteError:=False
        ' Write the error to the project log
        WriteError errUnableToLogOutput, gstrSource, mstrCommand
    Else
        strOutput = strOutput & " 2> " & mcErrorFile.FileName
        CloseFileSM (mcErrorFile.FileName)
    End If
        
    If Not StringEmpty(mstrStartDir) Then
        sOldDir = CurDir
        If Mid$(mstrStartDir, 2, 1) = ":" Then
            ChDrive mstrStartDir
        End If
        ChDir mstrStartDir
    End If
    
    ' If either the output or error file paths is invalid, we
    ' run the Shell command, but we do not generate the
    ' corresponding output
    
    If mblnRunAsyncSM Then
        ' Without start
        strExec = "cmd /c " & mstrCommand & strOutput
        BugMessage "Executing: " & strExec
        Set mcAsyncProc = New cAsyncShell
        mdtmStartTime = Now
        RaiseEvent Start(mdtmStartTime)
        Call mcAsyncProc.Shell(strExec)
    Else
        mdtmStartTime = Now
        RaiseEvent Start(mdtmStartTime)
        Call SyncShell(mstrCommand)
        Call ExecutionComplete
    End If
    
    If Not StringEmpty(sOldDir) Then
        If Mid$(sOldDir, 2, 1) = ":" Then
            ChDrive sOldDir
        End If
        ChDir sOldDir
    End If
    
End Sub
Public Property Get OutputFile() As String

    OutputFile = mcOutputFile.FileName
    
End Property
Public Property Let OutputFile(ByVal vdata As String)
    
    ' Initialize the filename property of the output file
    ' instance, since we use it for all write operations
    mcOutputFile.FileName = vdata

End Property
'Public Property Get LogFile() As String
'
'    LogFile = mcLogFile.FileName
'
'End Property
'Public Property Let LogFile(ByVal vdata As String)
'
'    ' Initialize the filename property of the log file
'    ' instance, since we use it for all write operations
'    mcLogFile.FileName = vdata
'
'    ' Initialize the message file on the RDO Errors object
'    mcRDOErrorsSM.MsgFile = vdata
'
'End Property
Public Property Get ErrorFile() As String

    ErrorFile = mcErrorFile.FileName
    
End Property
Public Property Get Command() As String

    Command = mstrCommand
    
End Property
Public Property Let ErrorFile(ByVal vdata As String)
    
    ' Initialize the filename property of the error file
    ' instance, since we use it for all write operations
    mcErrorFile.FileName = vdata

    ' Initialize the error file on the RDO Errors object
    mcRDOErrorsSM.ErrorFile = vdata
    mcVBErrorsSM.ErrorFile = vdata
    
End Property
Public Property Let Command(ByVal vdata As String)
    
    mstrCommand = vdata

End Property
Public Property Get ProjectName() As String

    ProjectName = mcRDOErrorsSM.ProjectName
    
End Property
Public Property Let ProjectName(ByVal vdata As String)
    
'    ' Initialize the project name on the RDO Errors object
'    mcRDOErrorsSM.ProjectName = vData
'    mcVBErrorsSM.ProjectName = vData
'
'    ' Initialize the project name on the output file
'    mcOutputFile.ProjectName = vData
'
End Property

Private Function ExecutionFailed() As Boolean
        
    Dim errRDOLoop As rdoError
    
    ExecutionFailed = False
    
    For Each errRDOLoop In rdoErrors
        Select Case errRDOLoop.SQLRetcode
            Case rdSQLSuccess, rdSQLSuccessWithInfo, rdSQLNoDataFound:
                
            Case Else:
                ExecutionFailed = True
                Exit For
        End Select
    Next

End Function

Private Sub Class_Initialize()

    StepStatus = gintPending

    Set mcRDOErrorsSM = New cRDOErrorsSM
    Set mcVBErrorsSM = New cVBErrorsSM
    
    ' Instantiate the file class for the output, error and
    ' log files
    Set mcErrorFile = New cFileSM
'    Set mcLogFile = New cFileSM
    Set mcOutputFile = New cFileSM
    
    ' Initialize the end time to mean empty
    mdtmEndTime = gdtmEmpty
    Set mcTimer = New cTimerSM

    mblnCancel = False
    
End Sub

Private Sub Class_Terminate()

    Set mcRDOErrorsSM = Nothing
    Set mcVBErrorsSM = Nothing
    
    ' Will close the log files
    Set mcErrorFile = Nothing
'    Set mcLogFile = Nothing
    Set mcOutputFile = Nothing
    
    Set mcTimer = Nothing
    
End Sub

Private Sub mcAsyncProc_Terminated()
        
    mdtmEndTime = Now
    StepStatus = gintComplete
    Call ExecutionComplete

End Sub
Private Sub mconConnection_QueryComplete(ByVal Query As RDO.rdoQuery, ByVal ErrorOccurred As Boolean)
'    Dim iError As ErrorStatus
'
'    On Error GoTo mconConnection_QueryCompleteErr
'
'    If mconConnection.StillExecuting = False Then
'        ' Initialize the end time for the query
'        mdtmEndTime = Now
'    End If
'
'    Call ProcessResultset
'
'    Exit Sub
'
'mconConnection_QueryCompleteErr:
'
'    ' Log all the errors generated by the ODBC layer, if any
'    mcRDOErrorsSM.LogRDOErrors
'
'    ' Log the VB errors
'    mcVBErrorsSM.LogVBErrors
'
End Sub
Private Sub mconConnection_QueryTimeout(ByVal Query As RDO.rdoQuery, Cancel As Boolean)

    ' mblnCancel will be set to True if an Abort operation is
    ' triggered
    Cancel = mblnCancel
    
End Sub

Private Sub mcTimer_Timer()

    On Error GoTo mcTimer_TimerErr
    
    If mblnCreateRs Then
        If Not mrecResults.StillExecuting Then
            mdtmEndTime = Now
            Call ProcessResultset
        End If
    Else
        If Not mconConnection.StillExecuting Then
            mdtmEndTime = Now
            Call ProcessResultset
        End If
    End If

    If mconConnection Is Nothing Then
        mcTimer.Enabled = False
    End If
    
    Exit Sub

mcTimer_TimerErr:

    ' Log the VB errors
    mcVBErrorsSM.LogVBErrors
    
    mdtmEndTime = Now
    mcTimer.Enabled = False
    ProcessResultset

End Sub

#End If
