VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cSequence"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'  FILE:        cSequence.cls
'               Microsoft TPC-H Kit Ver. 2.7.0-1005
'               Copyright Microsoft, 2008
'               All Rights Reserved
'
'
'  PURPOSE:     This class uses the att_identifiers table to generate unique
'               identifiers.
'  Contact:     Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit

Private mlngIdentifier As Long
Private mstrIdentifierColumn As String
Private mrecIdentifiers As Recordset
Private mdbsDatabase As Database

Private Const mstrEmptyString = ""

' Used to indicate the source module name when errors
' are raised by this class
Private mstrSource As String
Private Const mstrModuleName As String = "cSequence."

Private Sub CreateIdRecord()
    ' Creates a record with all identifiers having an initial value of 1

    Dim sSql As String
    Dim pId As DAO.Parameter
    Dim qyId As DAO.QueryDef
    
    sSql = "insert into att_identifiers (" & _
            " workspace_id, parameter_id, step_id, " & _
            " constraint_id, run_id, connection_id " & _
            ", " & FLD_ID_CONN_NAME & _
            " ) values ( " & _
            "[w_id], [p_id], [s_id], [c_id], [r_id], [conn_id], [conn_dtl_id] )"
    Set qyId = mdbsDatabase.CreateQueryDef(gstrEmptyString, sSql)
    For Each pId In qyId.Parameters
        pId.Value = glMinId
    Next pId
    qyId.Execute dbFailOnError
    qyId.Close
    
End Sub
Private Sub CreateIdRecordset()

    Dim strSql As String
    
    ' Initialize the recordset with all identifiers
    strSql = "select * from att_identifiers"
    Set mrecIdentifiers = mdbsDatabase.OpenRecordset(strSql, dbOpenForwardOnly)
    
    If mrecIdentifiers.RecordCount = 0 Then
        CreateIdRecord
        Set mrecIdentifiers = mdbsDatabase.OpenRecordset(strSql, dbOpenForwardOnly)
    End If

    BugAssert mrecIdentifiers.RecordCount <> 0

End Sub

Public Property Set IdDatabase(vdata As Database)

    Set mdbsDatabase = vdata
    
End Property

Public Property Let IdentifierColumn(vdata As String)

    Dim intIndex As Integer
    
    On Error GoTo IdentifierColumnErr
    
    ' Initialize the return value to an empty string
    mstrIdentifierColumn = mstrEmptyString
    Call CreateIdRecordset
    
    For intIndex = 0 To mrecIdentifiers.Fields.Count - 1
        
        If LCase(Trim(mrecIdentifiers.Fields(intIndex).Name)) = _
                LCase(Trim(vdata)) Then
            
            ' Valid column name
            mstrIdentifierColumn = vdata
            Exit Property
        End If
        
    Next intIndex
    
    BugAssert True, "Invalid column name!"
    
    Exit Property
    
IdentifierColumnErr:
    LogErrors Errors
    mstrSource = mstrModuleName & "IdentifierColumn"
    On Error GoTo 0
    Err.Raise vbObjectError + errIdentifierColumnFailed, _
            mstrSource, _
            LoadResString(errIdentifierColumnFailed)
    
End Property
Public Property Get Identifier() As Long
    Dim strSql As String
        
    On Error GoTo GetIdentifierErr
    
    BugAssert mstrIdentifierColumn <> mstrEmptyString
    
    ' Increment the identifier column by 1
    strSql = "update att_identifiers " & _
        " set " & mstrIdentifierColumn & _
        " = " & mstrIdentifierColumn & " + 1"
    mdbsDatabase.Execute strSql, dbFailOnError
    
    ' Refresh the recordset with identifier values
    Call CreateIdRecordset
    
    mlngIdentifier = mrecIdentifiers.Fields(mstrIdentifierColumn).Value

    Identifier = mlngIdentifier
    
    Exit Property
    
GetIdentifierErr:
    LogErrors Errors
    mstrSource = mstrModuleName & "Identifier"
    On Error GoTo 0
    Err.Raise vbObjectError + errGetIdentifierFailed, _
            mstrSource, _
            LoadResString(errGetIdentifierFailed)
    
End Property
Private Sub Class_Terminate()

    mrecIdentifiers.Close
    
End Sub

